@@ -1,4 +1,3 @@
-import copy
 from os import times
 from sys import getswitchinterval, exit
 
@@ -11,6 +10,7 @@ import hashlib
 import aiohttp
 from aiohttp import web
 
+from gear.utils.compat import meter_log_convert_to_eth_log
 from .rpc import make_version
 from json.decoder import JSONDecodeError
 from .meter.account import (
@@ -34,7 +34,12 @@ res_headers = {
     "Connection": "keep-alive",
 }
 
+SUB_ID = '0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827'
 
+async def checkHealth(request, logging=False, debug=False):
+    r = {"jsonrpc":"2.0","method":"eth_blockNumber","params":[],"id":8545}
+    response = await async_dispatch(json.dumps(r), basic_logging=logging, debug=debug)
+    return web.json_response(response.deserialized(), headers=res_headers, status=response.http_status)
 
 async def handle(request, logging=False, debug=False):
     jreq = await request.json()
@@ -49,12 +54,16 @@ async def handle(request, logging=False, debug=False):
     print("call: [%s] ts:%.0f"% (jreq[0]['method'] if jreq and len(jreq)>=1 else 'unknown', datetime.now().timestamp()),
               "\nRequest:", reqStr)
     for r in jreq:
-        method = r['method']
         # request = await request.text()
         response = await async_dispatch(json.dumps(r), basic_logging=logging, debug=debug)
         if response.wanted:
-            print("Response #%s:"%(str(r['id'])), json.dumps(response.deserialized()))
+            # print("Response #%s:"%(str(r['id'])), json.dumps(response.deserialized()))
+            print("Response #%s: DONE"%(str(r['id'])))
             responses.append(json.loads(json.dumps(response.deserialized())))
+        if response.http_status != 200:
+            print("Error response #%s:" %(str(r['id'])), json.dumps(response.deserialized()))
+            print("HTTP_STATUS: ", response.http_status)
+
 
     print("-"*40)
     if len(responses):
@@ -134,7 +143,7 @@ def meter_block_convert_to_eth_block(block):
     else:
         block["nonce"] = encode_number(n, 8)
 
-    # TODO: fix "fake" sha3Uncles
+    # sha3Uncles is always empty on meter
     block['sha3Uncles'] = '0x1dcc4de8dec75d7aab85b567b6ccd41ad312451b948a7413f0a142fd40d49347'
     # TODO: fix "fake" transactions root
     if len(block['transactions']) ==0:
@@ -153,7 +162,8 @@ def meter_block_convert_to_eth_block(block):
 
     
 
-conns = {}
+newHeadListeners = {} # ws connection id -> ws connection
+logListeners = {} # ws connection id -> { ws: ws connection, filters: filters }
 # WSURL_NHEADS = 'ws://127.0.0.1:8669/subscriptions/beat'
 
 def hash_digest(param):
@@ -161,36 +171,91 @@ def hash_digest(param):
     digest = h.hexdigest()
     return digest
 
-async def run_ws_client(endpoint):
+async def run_new_head_observer(endpoint):
     ws_endpoint = endpoint.replace('https', 'ws').replace('http','ws')+'/subscriptions/beat'
-    async with websockets.connect(ws_endpoint) as beatWS:
-        async for msg in beatWS:
-            # print('got: ', msg)
-            for key in list(conns.keys()):
-                ws = conns[key]
-                r = json.loads(msg)
-                if r.get("number"):
-                    num = int(r.get("number"), 16)
-                    print("forward block %d to conn %s" %(num,key))
-                else:
-                    print('forward to conn', key)
+    while True:
+        try:
+            async with websockets.connect(ws_endpoint) as beatWS:
+                async for msg in beatWS:
+                    # print('got: ', msg)
+                    for key in list(newHeadListeners.keys()):
+                        ws = newHeadListeners[key]
+                        r = json.loads(msg)
+                        if r.get("number"):
+                            num = int(r.get("number"), 16)
+                            print("forward block %d to conn %s" %(num,key))
+                        else:
+                            print('forward to conn', key)
 
-                r['timestamp']  = hex(r['timestamp'])
-                r['gasLimit'] = hex(r['gasLimit']).replace('0x0', '')
-                r['gasUsed'] = hex(r['gasUsed']).replace('0x0','')
-                r['nonce'] = hex(r['nonce']).replace('0x0', '0x0000000000000000')
-                try:
-                    out = json.dumps({"jsonrpc": "2.0", "method":"eth_subscription" ,"params":{"subscription":"0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827", "result":r},})
-                    await ws.send_str(out)
-                except Exception as e:
-                    del conns[key]
-                    print('error happend for ', key, 'ignored', e)
+                        r['timestamp']  = hex(r['timestamp'])
+                        r['gasLimit'] = hex(r['gasLimit']).replace('0x0', '')
+                        r['gasUsed'] = hex(r['gasUsed']).replace('0x0','')
+                        r['nonce'] = hex(r['nonce']).replace('0x0', '0x0000000000000000')
+                        try:
+                            out = json.dumps({"jsonrpc": "2.0", "method":"eth_subscription" ,"params":{"subscription":SUB_ID, "result":r}})
+                            await ws.send_str(out)
+                        except Exception as e:
+                            del newHeadListeners[key]
+                            print('error happend for client ws', key, 'ignored', e)
+        except Exception as e:
+            print('error happend in head observer', e)
+            print('retry in 10 seconds')
+            await asyncio.sleep(10)
 
+def match_filter(log, filters):
+    for filter in filters:
+        addressMatch = True
+        topicsMatch = True
+        if 'address' in filter:
+            address = filter['address']
+            if address != log['address']:
+                addressMatch = False
+    
+        if 'topics' in filter and isinstance(filter['topics'], list) and len(filter['topics'])>0:
+            topics = filter['topics']
+            for index, topic in enumerate(topics):
+                if len(log['topics']) < index+1 or topic and topic != log['topics'][index]:
+                    topicsMatch = False
+                    break
+        if addressMatch and topicsMatch:
+            return True
+    return False
+        
+
+async def run_event_observer(endpoint):
+    ws_endpoint = endpoint.replace('https', 'ws').replace('http','ws')+'/subscriptions/event'
+    while True:
+        try:
+            async with websockets.connect(ws_endpoint) as eventWS:
+                async for msg in eventWS:
+                    # print('got: ', msg)
+                    for key in list(logListeners.keys()):
+                        info = logListeners[key]
+                        ws = info['ws']
+                        filters = info['filters']
+                        log = json.loads(msg)
+                        if not match_filter(log, filters):
+                            print('not match filter, skip now', "key:", key)
+                            continue
+                        result = meter_log_convert_to_eth_log(log)
+                        result['logIndex'] = result['logIndex'].decode('utf-8')
+                        result['transactionIndex'] = result['transactionIndex'].decode('utf-8')
+                        result['blockNumber'] = result['blockNumber'].decode('utf-8')
+                        try:
+                            out = json.dumps({"jsonrpc": "2.0", "method":"eth_subscription" ,"params":{"subscription":SUB_ID, "result":result}})
+                            await ws.send_str(out)
+                        except Exception as e:
+                            del logListeners[key]
+                            print('error happend for client ws', key, 'ignored', e)
+        except Exception as e:
+            print('error happend in event observer', e)
+            print('log:', log)
+            print('filters:', filters)
+            print('retry in 10 seconds')
+            await asyncio.sleep(10)
 
 async def websocket_handler(request):
-        count = 1
         headers = request.headers
-        # print("HEADERS: ", headers)
         if (
             headers.get("connection", "").lower() == "upgrade"
             and headers.get("upgrade", "").lower() == "websocket"
@@ -205,7 +270,6 @@ async def websocket_handler(request):
                     # print("REQ: ", msg.data)
                     if msg.type == aiohttp.WSMsgType.TEXT and msg.data.strip():
                         # if is a valid json request
-
                         jreq = json.loads(msg.data)
 
                         # handle batch requests
@@ -217,20 +281,31 @@ async def websocket_handler(request):
                             await ws.send_str(json.dumps(ress))
                             continue
 
-                        if 'method' not in jreq:
+                        if 'method' not in jreq or 'id' not in jreq:
                             # not a valid json request
                             continue
+                        
+                        id = jreq['id']
 
-                        if(jreq['method'] == "eth_subscribe"):
+                        if jreq['method'] == "eth_subscribe":
                             # handle subscribe
-                            print("SUBSCRIBE:", msg.data, "key:", key)
-                            if key in conns:
-                                continue
-                            conns[key] = ws
-                            print("appended %s to conns" % key)
-                            #send a subscription id to the client
-                            await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":"0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827", "id":jreq['id']}))
-                            
+                            if isinstance(jreq['params'], list):
+                                params = jreq['params']
+                                if params[0] == 'newHeads':
+                                    if key in newHeadListeners:
+                                        continue
+                                    newHeadListeners[key] = ws
+                                    print("SUBSCRIBE to newHead")
+                                    #send a subscription id to the client
+                                    await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":SUB_ID, "id":id}))
+                                
+                                if params[0] == 'logs':
+                                    if key in logListeners:
+                                        continue
+                                    logListeners[key] = {"ws":ws, "filters":params[1:]}
+                                    print("SUBSCRIBE to logs", "filters:", params[1:])
+                                    await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":SUB_ID, "id":id}))
+
                             #begin subscription
                             # while True:
                             
@@ -239,14 +314,15 @@ async def websocket_handler(request):
                             #     # convert the subscription object into an appropriate response
                             #     result = meter_block_convert_to_eth_block(copy_obj['result'])
                                 
-                            #     res_obj = {"jsonrpc": copy_obj["jsonrpc"] , "method":"eth_subscription", "params":{"result":result, "subscription":"0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827"}}
+                            #     res_obj = {"jsonrpc": copy_obj["jsonrpc"] , "method":"eth_subscription", "params":{"result":result, "subscription":SUB_ID}}
                             #     await ws.send_str(json.dumps(res_obj))
                         elif (jreq['method'] == "eth_unsubscribe"):
                             # handle unsubscribe
-                            await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":True, "id":jreq['id']}))
-                            if key not in conns:
-                                continue
-                            del conns[key]
+                            await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":True, "id":id}))
+                            if key in newHeadListeners:
+                                del newHeadListeners[key]
+                            if key in logListeners:
+                                del logListeners[key]
                             print("UNSUBSCRIBE key:", key)
                             await ws.close()
                         else:
@@ -266,9 +342,9 @@ async def websocket_handler(request):
                         await ws.pong()
                         
                     elif ws.closed:
-                        if key not in conns:
+                        if key not in newHeadListeners:
                             return
-                        del conns[key]
+                        del newHeadListeners[key]
                         await ws.close(code=ws.close_code, message=msg.extra)
                     else:
                         print("unknown REQ: ", msg)
@@ -305,7 +381,7 @@ def get_http_app(host, port, endpoint, keystore, passcode, log, debug, chainid):
     app.router.add_post("/", lambda r: handle(r, log, debug))
     app.router.add_options("/", lambda r: web.Response(headers=res_headers))
     app.router.add_get(
-        "/health", lambda r: web.Response(headers=res_headers, body="OK", content_type="text/plain"))
+        "/health", lambda r: checkHealth(r,log,debug))
     # web.run_app(app, host=host, port=port)
     return app
 
@@ -361,10 +437,11 @@ async def run_server(host, port, endpoint, keystore, passcode, log, debug, chain
     await ws.start()
     print("Websocket Listening on %s:%s" % (host, int(port)+1))
 
-    ws_client = asyncio.create_task(run_ws_client(endpoint))
-    await ws_client
-    while True:
-        await asyncio.sleep(3600)  # sleep forever
+    head_observer = asyncio.create_task(run_new_head_observer(endpoint))
+    event_observer = asyncio.create_task(run_event_observer(endpoint))
+    await head_observer
+    # while True:
+        # await asyncio.sleep(3600)  # sleep forever
 
 
 @click.command()
@@ -404,7 +481,11 @@ async def run_server(host, port, endpoint, keystore, passcode, log, debug, chain
     default="0x53"
 )
 def main(host, port, endpoint, keystore, passcode, log, debug, chainid):
-    asyncio.run(run_server(host, port, endpoint, keystore, passcode, log, debug, chainid))
+    chainIdHex = chainid
+    if not chainid.startswith('0x'):
+        chainIdHex = hex(int(chainid))
+
+    asyncio.run(run_server(host, port, endpoint, keystore, passcode, log, debug, chainIdHex))
 
     
 
