@@ -1,6 +1,6 @@
 import rlp
 import uuid
-import random
+# import random
 from gear.utils.singleton import Singleton
 from gear.utils.types import (
     encode_number,
@@ -21,9 +21,17 @@ from .request import (
     get,
     post,
 )
+from jsonrpcserver.exceptions import ApiError
 
 
-def _attribute(obj, key): return None if obj is None else obj[key]
+def _attribute(obj, key): 
+    if obj:
+        if key in obj:
+            return obj[key]
+        if 'error' in obj and 'code' in obj:
+            raise ApiError(obj['error'], obj['code'])
+        raise ApiError('not valid response from node', -2001, obj)
+    return None
 
 
 class MeterClient(object, metaclass=Singleton):
@@ -87,23 +95,30 @@ class MeterClient(object, metaclass=Singleton):
         blk = await self.blocks("best").make_request(get)
         return _attribute(blk, "number")
 
+    async def get_syncing(self):
+        blk = await self.blocks("best").make_request(get)
+        num = encode_number(_attribute(blk, 'number'))
+        return {"startingBlock":num, "currentBlock": num, "highestBlock": num}
+
     async def get_block_id(self, block_identifier):
         blk = await self.blocks(block_identifier).make_request(get)
         return _attribute(blk, "id")
 
     async def estimate_gas(self, transaction):
         data = {
-            "data": transaction["data"],
+            "data": transaction.get("data",'0x'),
             "value": (encode_number(transaction.get("value", 0))).decode("utf-8"),
             "caller": transaction.get("from", None),
         }
-        print("Data:", data)
         toAddr = transaction.get("to", "0x")
         toAddr = "0x" if toAddr == None else toAddr
-        print("To Addr")
         result = await self.accounts(toAddr).make_request(post, data=data)
-        if result is None or result["reverted"]:
-            raise ValueError("Gas estimation failed.")
+        if result is None:
+            print("empty response, estimate gas with data:", data)
+            raise ApiError('gas estimation error: no response from server',-1000)
+        if result["reverted"]:
+            print("reverted, estimate gas with data:", data)
+            raise ApiError('gas estimation error: '+result['vmError'], -1001)
         return int(result["gasUsed"] * 1.2) + intrinsic_gas(transaction)
 
     async def call(self, transaction, block_identifier):
@@ -111,7 +126,7 @@ class MeterClient(object, metaclass=Singleton):
             "revision": block_identifier,
         }
         data = {
-            "data": transaction["data"],
+            "data": transaction.get("data", "0x"),
             "value": (encode_number(transaction.get("value", 0))).decode("utf-8"),
             "caller": transaction.get("from", None),
         }
@@ -184,13 +199,10 @@ class MeterClient(object, metaclass=Singleton):
         result = []
         step = 100
         if isinstance(address, list) and len(address) > step:
-            print('get logs with loop')
             while len(address)>0:
                 result.extend(await self.get_logs_bounded(address[:step],query))
                 address = address[step:]
-                print('len:',len(address))
         else:
-            print('get logs with direct call')
             result = await self.get_logs_bounded(address, query)
         return result
 
@@ -200,28 +212,21 @@ class MeterClient(object, metaclass=Singleton):
                 query['address'] = address
             elif isinstance(address, str):
                 query['address'] = [address]
-        print("POST to /logs/events: ", query)
         logs = await self.events.make_request(post, data=query)
-        result = meter_log_convert_to_eth_log(address, logs)
+        result = meter_log_convert_to_eth_log(logs)
         return result
 
 
     async def get_trace_filter(self, filter_obj):
-        print("filter_obj", filter_obj)
         result = await self.debug.openeth_trace_filter.make_request(post, data=filter_obj)
-        print("trace_filter result", result)
         return result
 
     async def get_trace_transaction(self, filter_obj):
-        print("filter_obj", filter_obj, type(filter_obj))
         result = await self.debug.openeth_trace_transaction.make_request(post, data=[filter_obj])
-        print("trace_transaction result", result)
         return result
 
     async def get_trace_block(self, filter_obj):
-        print("filter_obj", filter_obj, type(filter_obj))
         result = await self.debug.openeth_trace_block.make_request(post, data=[filter_obj])
-        print("trace_block result", result)
         return result
 class BlockFilter(object):
 
