@@ -1,6 +1,7 @@
 import itertools
 import functools
 import logging
+from sqlite3 import apilevel
 import sys
 import json
 import traceback
@@ -20,6 +21,7 @@ from .utils.types import (
     normalize_number
 )
 from jsonrpcserver import method
+from jsonrpcserver.exceptions import ApiError
 
 
 
@@ -204,12 +206,23 @@ async def eth_getCode(address, block_identifier="best"):
 async def eth_blockNumber():
     return encode_number(await meter.get_block_number())
 
+@method
+@async_serialize
+async def eth_syncing():
+    return await meter.get_syncing()
+
+@method
+@async_serialize
+async def eth_maxPriorityFeePerGas():
+    return encode_number(0)
+
+
 
 @method
 @async_serialize
 async def eth_estimateGas(transaction):
     formatted_transaction = input_transaction_formatter(transaction)
-    print("FORMATED TX:", formatted_transaction)
+    # print("FORMATED TX:", formatted_transaction)
     result = await meter.estimate_gas(formatted_transaction)
     return encode_number(result)
 
@@ -317,7 +330,6 @@ async def eth_getLogs(filter_obj):
     if (to_blk == 'latest'):
         latest = await meter.get_block('best')
         filter_obj['toBlock'] = latest['number']
-    print(filter_obj)
     return await meter.get_logs(filter_obj.get("address", None), input_log_filter_formatter(filter_obj))
 
 @method
