@@ -50,7 +50,7 @@ def meter_block_convert_to_eth_block(block):
     else:
         block["nonce"] = encode_number(n, 8)
 
-    # TODO: fix "fake" sha3Uncles
+    # sha3Uncles is always empty on meter
     block['sha3Uncles'] = '0x1dcc4de8dec75d7aab85b567b6ccd41ad312451b948a7413f0a142fd40d49347'
     # TODO: fix "fake" transaction root
     if len(block['transactions'])==0:
@@ -109,23 +109,37 @@ def meter_receipt_log_convert_to_eth_log(receipt, index, log):
     }
 
 
-def meter_log_convert_to_eth_log(address, logs):
-    if logs:
+def meter_log_convert_to_eth_log(logs):
+    print(logs)
+    if logs and isinstance(logs,list):
         return [
             {
                 "address": log["address"],
-                "logIndex": encode_number(index),
-                "blockNumber":encode_number(log["meta"]["blockNumber"]) ,
+                "logIndex": encode_number(log["logIndex"]),
+                "blockNumber":encode_number(log["meta"]["blockNumber"]),
                 "blockHash": log["meta"]["blockID"],
                 "transactionHash": log["meta"]["txID"],
                 "transactionIndex": encode_number(0),
                 # "address": address,
                 "data": log["data"],
                 "topics": log["topics"],
-		"removed": False
-}
+                "removed": False
+            }
             for index, log in enumerate(logs)
         ]
+    if logs and isinstance(logs, object):
+        return {
+            "address": logs["address"],
+            "logIndex": encode_number(logs["logIndex"]),
+            "blockNumber":encode_number(logs["meta"]["blockNumber"]),
+            "blockHash": logs["meta"]["blockID"],
+            "transactionHash": logs["meta"]["txID"],
+            "transactionIndex": encode_number(0),
+            # "address": address,
+            "data": logs["data"],
+            "topics": logs["topics"],
+            "removed": False
+        }
     return []
 
 
@@ -133,9 +147,9 @@ def meter_log_convert_to_eth_log(address, logs):
 # transaction
 #
 def meter_tx_convert_to_eth_tx(tx):
-    r = '0x'
-    s = '0x'
-    v = '0x'
+    r = '0x0000000000000000000000000000000000000000000000000000000000000000'
+    s = '0x0000000000000000000000000000000000000000000000000000000000000000'
+    v = '0x00'
     if tx["ethTx"] and tx["ethTx"]["r"] and tx["ethTx"]["s"] and tx["ethTx"]["v"]:
         r = tx["ethTx"]["r"]
         s = tx["ethTx"]["s"]
@@ -255,7 +269,7 @@ def data_gas(data):
 
 def intrinsic_gas(transaction):
     total = TX_GAS
-    gas = data_gas(transaction["data"])
+    gas = data_gas(transaction.get("data", "0x"))
     total += gas
     cgas = CLAUSE_GAS
     if "to" not in transaction:
